/*
 * File: usb_msc.c
 */
#include <stdlib.h>
#include <stdbool.h>

#include "usb_cfg.h"

#if( USB_CFG_MSC_USE )

#include "usb_msc.h"

#define CBD_TEST_UNIT_READY_CMD             ( 0x00 )

#define CBD_REQUEST_SENSE_CMD               ( 0x03 )

#define CBD_INQUIRY_CMD                     ( 0x12 )

#define CBD_MODE_SENSE_6_CMD                ( 0x1A )

#define CBD_STOP_START_CMD                  ( 0x1B )

#define CBD_READ_FORMAT_CAPACITY_CMD        ( 0x23 )

#define CBD_READ_CAPACITY_CMD               ( 0x25 )

#define CBD_READ_10_CMD                     ( 0x28 )

#define CBD_WRITE_CMD                       ( 0x2A )

#define CBD_VERIFY_CMD                      ( 0x2F )


struct __attribute__((packed)) MODE_SENSE_6_BYTE_1_BITS
{
    uint8_t RSVRD1      :       3;

    uint8_t DBD         :       1;

    uint8_t RSVRD2      :       4;
};

union __attribute__((packed)) MODE_SENSE_6_BYTE_1_REG
{
    uint8_t all;

    struct MODE_SENSE_6_BYTE_1_BITS bit;
};


struct __attribute__((packed)) MODE_SENSE_6_BYTE_2_BITS
{
    uint8_t PAGE_CODE   :       6;

    uint8_t PC          :       2;

};

union __attribute__((packed)) MODE_SENSE_6_BYTE_2_REG
{
    uint8_t all;

    struct MODE_SENSE_6_BYTE_2_BITS bit;
};

struct __attribute__((packed)) MODE_SENSE_6_CMD
{
    uint8_t OPERATION_CODE;

    union MODE_SENSE_6_BYTE_1_REG   MODE_SENSE_6_BYTE_1;

    union MODE_SENSE_6_BYTE_2_REG   MODE_SENSE_6_BYTE_2;

    uint8_t SUBPAGE_CODE;

    uint8_t ALLOCATION_LENGTH;

    uint8_t CONTROL;
};


USB_DEVICE_DESCRIPTOR_STRUCT USB_DEVICE_DEVICE_DESC = {
    .bLength            =   sizeof( USB_DEVICE_DESCRIPTOR_STRUCT ), //0x12,

    .bDescriptorType    =   DEVICE_DESCRIPTOR_TYPE,

    .bcdUSB             =   0x0200,

    .bDeviceClass       =   0,

    .bDeviceSubclass    =   0,

    .bDeviceProtocol    =   0,

    .bMaxPacketSize0    =   0x40,

    .idVendor           =   0x04D8,

    .iProduct           =   0x0009,

    .bcdDevice          =   0x0100,

    .iManufacturer      =   0x01,

    .iProduct           =   0x02,

    .iSerialNumber      =   0x03,

    .bNumConfigurations =   0x01,
};


USB_CONFIGURATION_DESCRIPTOR_STRUCT USB_DEVICE_CONFIGURATION_DESCs[] = {

    {
        .bLength                =   sizeof( USB_CONFIGURATION_DESCRIPTOR_STRUCT ), //0x09,

        .bDescriptorType        =   CONFIGURATION_DESCRIPTOR_TYPE,

        .wTotalLength           =   ( sizeof(USB_CONFIGURATION_DESCRIPTOR_STRUCT ) +
                                      sizeof( USB_INTERFACE_DESCRIPTOR_STRUCT )    +
                                      ( 2 * sizeof( USB_ENDPOINT_DESCRIPTOR_STRUCT ) )
                                    ), //0x0020,

        .bNumInterfaces         =   0x01,

        .bConfigurationValue    =   0x01,

        .iConfiguration         =   0x00,

        .bmAttributes           =   0xC0,   // Attributes: self powered, no remote wakeup

        .bMaxPower              =   0x32,   // Maximum power consumption (100 mA)
    },
};



USB_STRING_DESCRIPTOR_STRUCT USB_STRING_DESC_0 = {
    .bLength                =   0x04,

    .bDescriptorType        =   STRING_DESCRIPTOR_TYPE,

    .wLANGID                =   USB_LANG_EN_US,
};


USB_INTERFACE_DESCRIPTOR_STRUCT USB_DEVICE_INTERFACE_DESCs[] = {

    {
        .bLength                =   0x09,

        .bDescriptorType        =   INTERFACE_DESCRIPTOR_TYPE,

        .bInterfaceNumber       =   0x00,

        .bAlternateSetting      =   0x00,

        .bNumEndpoints          =   0x02,

        .bInterfaceClass        =   USB_CLASS_MASS_STORAGE,

        .bInterfaceSubclass     =   USB_SUBCLASS_SCSI_TRANSPARENT_CMD_SET,

        .bInterfaceProtocol     =   USB_BULK_ONLY_INTERFACE_PROTOCOL,

        .iInterface             =   0,
    },
};


USB_ENDPOINT_DESCRIPTOR_STRUCT USB_DEVICE_ENDPOINT_DESCs[] = {

    {
         .bLength                =   0x07,

         .bDescriptorType        =   ENDPOINT_DESCRIPTOR_TYPE,

         .bEndpointAddress       =   0x81,       //Endpoint number and direction (1 IN)

         .bmAttributes           =   0x02,       // Transfer type (bulk)

         .wMaxPacketSize         =   0x0040,     // Maximum packet size (64)

         .bInterval              =   0x00,
    },

    {
         .bLength                =   0x07,

         .bDescriptorType        =   ENDPOINT_DESCRIPTOR_TYPE,

         .bEndpointAddress       =   0x01,       //Endpoint number and direction (1 OUT)

         .bmAttributes           =   0x02,       // Transfer type (bulk)

         .wMaxPacketSize         =   0x0040,     // Maximum packet size (64)

         .bInterval              =   0x00,
    },
};


typedef enum
{
    IDLE_TRANSPORT_PHASE,

    DATA_TRANSPORT_PHASE,

    STATUS_TRANSPORT_PHASE,

    ERROR_TRANSPORT_PHASE,

} MSC_BLOCK_ONLY_TRANSPORT_PHASE_ENUM;


static MSC_BLOCK_ONLY_TRANSPORT_PHASE_ENUM sTansnportPhase;


static volatile CSW_STRUCT   CSWReply = {
    .dCSWSignature = CSW_SIGNATURE,
};

/*
 * The host communicates its intent to the device through the CBW. The device performs two verifications on
 * every CBW received. First, the device verifies that what was received is a valid CBW. Next, the device
 * determines if the data within the CBW is meaningful.
 * The device shall not use the contents of the dCBWTag in any way other than to copy its value to the dCSWTag of
 * the corresponding CSW.
 *
 * The device shall consider the CBW valid when:
 *  1. The CBW was received after the device had sent a CSW or after a reset,
 *  2. The CBW is 31 (1Fh) bytes in length,
 *  3. And the dCBWSignature is equal to 43425355h.
 *
 * The device shall consider the contents of a valid CBW meaningful when:
 * 1. No reserved bits are set,
 * 2. The bCBWLUN contains a valid LUN supported by the device,
 * 3. And both bCBWCBLength and the content of the CBWCB are in accordance with bInterfaceSubClass.
 *
 */
static bool isCSWDalid( CBW_STRUCT* pCSW )
{
    bool isValid = false;

    if( sTansnportPhase == IDLE_TRANSPORT_PHASE )
    {
        if( pCSW->dCBWSignature == CBW_SIGNATURE )
        {
            CSWReply.dCSWTag            = pCSW->dCBWTag;

            CSWReply.dCSWDataResidue    = pCSW->dCBWDataTransferLength;

            CSWReply.bCSWStatus         = 0x00;

            isValid = true;
        }
    }

    return ( isValid );
}


static void process_mode_sence_6( CBW_STRUCT* pCBW )
{
    struct MODE_SENSE_6_CMD* pMSCmd;

    pMSCmd = ( ( struct MODE_SENSE_6_CMD* )( &pCBW->CBWCB ) );
}



static void process_cbd( CBW_STRUCT* pCBW )
{
    switch( pCBW->CBWCB[0] )
    {
        case CBD_TEST_UNIT_READY_CMD:
        {

            break;
        }
        case CBD_REQUEST_SENSE_CMD:
        {

            break;
        }
        case CBD_INQUIRY_CMD:
        {

            break;
        }
        case CBD_MODE_SENSE_6_CMD:
        {

            break;
        }
        case CBD_STOP_START_CMD:
        {

            break;
        }
        case CBD_READ_FORMAT_CAPACITY_CMD:
        {

            break;
        }
        case CBD_READ_CAPACITY_CMD:
        {

            break;
        }
        case CBD_READ_10_CMD:
        {

            break;
        }
        case CBD_WRITE_CMD:
        {

            break;
        }
        case CBD_VERIFY_CMD:
        {

            break;
        }
        default:
        {
            // Unsupported command
            CSWReply.dCSWDataResidue    = 0;

            CSWReply.bCSWStatus         = 0x01;

            break;
        }
    }
}



void usb_msc_process_cbw( void* pRcvMsg )
{
    CBW_STRUCT* pCBW;

    pCBW = ( ( CBW_STRUCT*)( pRcvMsg ) );

    if( pRcvMsg != NULL )
    {
        if( isCSWDalid( pCBW ) )
        {
            process_cbd( pCBW );
        }
    }
}


#endif      /*  USB_CFG_MSC_USE  */


