/*
 * File: usb_control.c
 */
#include <stdlib.h>

#include "F28M35H52C1_Device.h"

#include "usb_device.h"

#include "usb_control.h"


/*
 * A data packet that contains a Data PID and error-checking bits but no data
 * bytes is a zero-length packet (ZLP). A ZLP can indicate the end of a
 * transfer or successful completion of a control transfer.
 */

/*
 * STANDARD REQUESTS
 * Request      Request Name            Target                              wValue                          wIndex                                      Data Statge
 * Code                                                                                                                                                 Data Source     wLength
 *                                                                                                                                                                      Contents
 * 0x00         Get Status              device, interface or endpoint       0x0000                          0x0000 (device), interface or endpoint      device          0x0002
 *
 * 0x01         Clear Feature           device, interface or endpoint       feature                         0x0000 (device), interface or endpoint      none            0x0000
 *
 * 0x03         Set Feature             device, interface or endpoint       feature                         0x0000 (device), interface or endpoint      none            0x0000
 *
 * 0x05         Set Address             device                              device address                  0x0000                                      none            0x0000
 *
 * 0x06         Get Descriptor          device                              descriptor type and index       0x0000 or language ID                       device          descriptor length (bytes);
 *                                                                                                                                                                      Descriptor
 * 0x07         Set Descriptor          device                              descriptor type and index       0x0000 or language ID                       host            descriptor length (bytes);
 *                                                                                                                                                                      Descriptor
 * 0x08         Get Configuration       device                              0x0000                          0x0000                                      device          0x0001;
 *                                                                                                                                                                      Configuration
 * 0x09         Set Configuration       device                              0x0000                          0x0000                                      none            0x0000
 *
 * 0x0A         Get Interface           interface                           interface                       interface                                   device          0x0001
 *                                                                                                                                                                      Alternate setting
 * 0x0B         Set Interface           interface                           interface                       interface                                   none            0x000
 *
 * 0x0C         Sync Frame              endpoint                            endpoint                        endpoint                                    device          0x0002
 *                                                                                                                                                                      Frame Number
 */


/*
 * Set Address
 * Purpose: The host specifies an address to use in future communications with
 * the device.
 * Request Number (bRequest): 05h
 * Source of Data: no Data stage
 * Data Length (wLength): 0000h
 * Contents of wValue field: new device address. Allowed values are 0001h007Fh.
 * Each device on the bus, including the root hub, has a unique address.
 * Contents of wIndex field: 0000h
 * Supported States: Default, Address.
 */
void usb_control_standard_request_device_set_address( void )
{
    ControlPacket.u8AssignedPendingAddress = ( ( USB_SETUP_PACKET_STRUCT* )( ControlPacket.RxTxBuf ) )->wValue.WB.L;

    ControlPacket.u8AssignedPendingAddress |= 0x80;

    ControlPacket.PHASE             = USB_CONTROL_TRANSFER_DATA_IN_PHASE;

    ControlPacket.u16TransferredLen = 0;

    ControlPacket.u16RequestLen     = 0;

    ControlPacket.u16PayloadLen     = 0;

    USBWriteEndPoint0Data();
}



/*
 * Get Descriptor
 * --------------
 * Purpose: The host requests a specific descriptor.
 * Request Number (bRequest): 06h
 * Source of Data: device
 * Data Length (wLength):
 *  The number of bytes to return. If the descriptor is longer than wLength,
 *  the device returns up to wLength bytes. If the descriptor is shorter than
 *  wLength, the device returns the entire descriptor.
 *  If the descriptor is shorter than wLength and is an even multiple of the
 *  endpoints maximum packet size, the device returns a ZLP in response to a
 *  request for more data after the device has sent the descriptor. The host
 *  detects the end of the data on receiving either the requested amount of data
 *  or a data packet containing less than the maximum packet size (including
 *  a ZLP).
 * Contents of wValue field:
 *  High byte: descriptor type.
 *  Low byte: descriptor index, to specify which descriptor to return when
 *            there are multiple descriptors of the same type.
 * Contents of wIndex field:
 *  For String descriptors, Language ID.
 *  Otherwise 0000h.
 * Contents of data packet in the Data stage:
 *  The requested descriptor.
 * Supported states:
 *  Default, Address, Configured.
 * Behavior on error:
 *  A device that doesnt support the specified descriptor should return STALL.
 */
static void usb_control_standard_request_device_descriptor( void )
{
    uint8_t u8DescriptorType;

    uint8_t* psrc;

    uint16_t u16Idx, u16Len, u16RequestLen;

    u8DescriptorType    = ( ( USB_SETUP_PACKET_STRUCT* )( ControlPacket.RxTxBuf ) )->wValue.WB.H;

    u16RequestLen       = ( ( USB_SETUP_PACKET_STRUCT* )( ControlPacket.RxTxBuf ) )->wLength;

    psrc = NULL;

    switch( u8DescriptorType )
    {
        case DEVICE_DESCRIPTOR_TYPE:
        {
            psrc = ( ( uint8_t* )( &USB_DEVICE_DEVICE_DESC ) );

            u16Len = USB_DEVICE_DEVICE_DESC.bLength;

            break;
        }
    }

    if( psrc != NULL )
    {
        for( u16Idx = 0; u16Idx < u16Len; u16Idx++ )
        {
            ControlPacket.RxTxBuf[ u16Idx ] = *psrc++;
        }

        ControlPacket.PHASE             = USB_CONTROL_TRANSFER_DATA_IN_PHASE;

        ControlPacket.u16TransferredLen = 0;

        ControlPacket.u16RequestLen     = u16RequestLen;

        ControlPacket.u16PayloadLen     = u16Len;

        USBWriteEndPoint0Data();
    }
}


static void usb_control_standard_request_device( void )
{
    switch( ( ( USB_SETUP_PACKET_STRUCT* )( ControlPacket.RxTxBuf ) )->bRequest )
    {
        case USB_REQUEST_SET_ADDRESS:
        {
            usb_control_standard_request_device_set_address( );

            break;
        }
        case USB_REQUEST_GET_DESCRIPTOR:
        {
            usb_control_standard_request_device_descriptor( );

            break;
        }
    }
}



static void usb_control_standard_request( void )
{
    switch( ( ( USB_SETUP_PACKET_STRUCT* )( ControlPacket.RxTxBuf ) )->bmRequestType.BM.Recipient )
    {
        case REQUEST_TO_DEVICE:
        {
            usb_control_standard_request_device( );

            break;
        }
    }
}


void usb_control_packet_process( void )
{
    switch( ( ( USB_SETUP_PACKET_STRUCT* )( ControlPacket.RxTxBuf ) )->bmRequestType.BM.Type )
    {
        case REQUEST_STANDARD:
        {
            usb_control_standard_request( );

            break;
        }
    }
}
