/*
 * File: usb.h
 */

#ifndef __USB_H__
#define __USB_H__

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

#define USB_LANG_EN_US                      ( 0x0409 )      // English (United States)


#define USB_CLASS_DEVICE                    ( 0x00 )
#define USB_CLASS_AUDIO                     ( 0x01 )
#define USB_CLASS_CDC                       ( 0x02 )
#define USB_CLASS_HID                       ( 0x03 )
#define USB_CLASS_PHYSICAL                  ( 0x05 )
#define USB_CLASS_IMAGE                     ( 0x06 )
#define USB_CLASS_PRINTER                   ( 0x07 )
#define USB_CLASS_MASS_STORAGE              ( 0x08 )
#define USB_CLASS_HUB                       ( 0x09 )
#define USB_CLASS_CDC_DATA                  ( 0x0a )
#define USB_CLASS_SMART_CARD                ( 0x0b )
#define USB_CLASS_SECURITY                  ( 0x0d )
#define USB_CLASS_VIDEO                     ( 0x0e )
#define USB_CLASS_HEALTHCARE                ( 0x0f )
#define USB_CLASS_DIAG_DEVICE               ( 0xdc )
#define USB_CLASS_WIRELESS                  ( 0xe0 )
#define USB_CLASS_MISC                      ( 0xef )
#define USB_CLASS_APP_SPECIFIC              ( 0xfe )
#define USB_CLASS_VEND_SPECIFIC             ( 0xff )
#define USB_CLASS_EVENTS                    ( 0xffffffff )


#define USB_SUBCLASS_SCSI_TRANSPARENT_CMD_SET       ( 0x06 )


#define USB_BULK_ONLY_INTERFACE_PROTOCOL            ( 0x50 )

typedef union __attribute__((packed)) _WORD_BYTE
{
    uint16_t W;

    struct {
        uint8_t L;

        uint8_t H;
    } WB;
} WORD_BYTE;


/* bmRequestType.Dir */
#define REQUEST_HOST_TO_DEVICE     0
#define REQUEST_DEVICE_TO_HOST     1

/* bmRequestType.Type */
#define REQUEST_STANDARD           0
#define REQUEST_CLASS              1
#define REQUEST_VENDOR             2
#define REQUEST_RESERVED           3

/* bmRequestType.Recipient */
#define REQUEST_TO_DEVICE          0
#define REQUEST_TO_INTERFACE       1
#define REQUEST_TO_ENDPOINT        2
#define REQUEST_TO_OTHER           3

/* bmRequestType Definition */
typedef union __attribute__((packed)) _REQUEST_TYPE
{
  struct _BM {
    uint8_t Recipient : 5;
    uint8_t Type      : 2;
    uint8_t Dir       : 1;
  } BM;

  uint8_t B;

} REQUEST_TYPE;

/* bRequest. USB Standard Request Codes */
#define USB_REQUEST_GET_STATUS                 0
#define USB_REQUEST_CLEAR_FEATURE              1
#define USB_REQUEST_SET_FEATURE                3
#define USB_REQUEST_SET_ADDRESS                5
#define USB_REQUEST_GET_DESCRIPTOR             6
#define USB_REQUEST_SET_DESCRIPTOR             7
#define USB_REQUEST_GET_CONFIGURATION          8
#define USB_REQUEST_SET_CONFIGURATION          9
#define USB_REQUEST_GET_INTERFACE              10
#define USB_REQUEST_SET_INTERFACE              11
#define USB_REQUEST_SYNC_FRAME                 12


/* USB GET_STATUS Bit Values */
#define USB_GETSTATUS_SELF_POWERED             0x01
#define USB_GETSTATUS_REMOTE_WAKEUP            0x02
#define USB_GETSTATUS_ENDPOINT_STALL           0x01

/* USB Standard Feature selectors */
#define USB_FEATURE_ENDPOINT_STALL             0
#define USB_FEATURE_REMOTE_WAKEUP              1

typedef enum
{
    USB_DISCONNECTED_STATE = 0,

    USB_POWERED_STATE,

    USB_DEFAULT_STATE,

    USB_ADDRESS_STATE,

    USB_CONFIGURED_STATE,

    USB_SUSPEND_STATE,

} USE_DEVICE_STATE_ENUM;


/* USB Default Control Pipe Setup Packet */
typedef struct __attribute__((packed)) _USB_SETUP_PACKET
{
  REQUEST_TYPE bmRequestType;

  uint8_t      bRequest;

  WORD_BYTE    wValue;

  WORD_BYTE    wIndex;

  uint16_t     wLength;

} USB_SETUP_PACKET_STRUCT;



typedef enum
{
    DEVICE_DESCRIPTOR_TYPE = 1,

    CONFIGURATION_DESCRIPTOR_TYPE,

    STRING_DESCRIPTOR_TYPE,

    INTERFACE_DESCRIPTOR_TYPE,

    ENDPOINT_DESCRIPTOR_TYPE,

} DESCRIPTOR_TYPE_ENUM;



typedef struct __attribute__((packed))
{
    uint8_t         bLength;            // Descriptor size in bytes (12h)

    uint8_t         bDescriptorType;    // The constant DEVICE (01h)

    uint16_t        bcdUSB;             // USB specification release number (BCD)

    uint8_t         bDeviceClass;       // Class code

    uint8_t         bDeviceSubclass;    // Subclass code

    uint8_t         bDeviceProtocol;    // Protocol Code

    uint8_t         bMaxPacketSize0;    // Maximum packet size for endpoint zero

    uint16_t        idVendor;           // Vendor ID

    uint16_t        idProduct;          // Product ID

    uint16_t        bcdDevice;          // Device release number (BCD)

    uint8_t         iManufacturer;      // Index of string descriptor for the manufacturer

    uint8_t         iProduct;           // Index of string descriptor for the product

    uint8_t         iSerialNumber;      // Index of string descriptor for the serial number

    uint8_t         bNumConfigurations; // Number of possible configurations

} USB_DEVICE_DESCRIPTOR_STRUCT;


typedef struct __attribute__((packed))
{
    uint8_t bLength;                       // Descriptor size in bytes (09h)

    uint8_t bDescriptorType;               // The constant CONFIGURATION (02h)

    uint16_t wTotalLength;                 // The number of bytes in the configuration

                                           // descriptor and all of its subordinate descriptors

    uint8_t bNumInterfaces;                // Number of interfaces in the configuration

    uint8_t bConfigurationValue;           // Identifier for Set Configuration and Get
                                           // Configuration requests

    uint8_t iConfiguration;                // Index of string descriptor for the configuration

    uint8_t bmAttributes;                  // Self/bus power and remote wakeup settings

    uint8_t bMaxPower;                     // Bus power required in units of 2 mA (USB 2.0) or 8
                                           // mA (SuperSpeed).

} USB_CONFIGURATION_DESCRIPTOR_STRUCT;


typedef struct __attribute__((packed))
{
    uint8_t bLength;                    // Descriptor size in bytes (variable)

    uint8_t bDescriptorType;            // The constant STRING (03h)

    uint16_t wLANGID;                   // For string descriptor zero, an array of one or more
                                        // Language Identifier codes. For other string
} USB_STRING_DESCRIPTOR_STRUCT;



typedef struct __attribute__((packed))
{
    uint8_t bLength;                       // Descriptor size in bytes (09h)

    uint8_t bDescriptorType;               // The constant INTERFACE (04h)

    uint8_t bInterfaceNumber;              // Number identifying this interface

    uint8_t bAlternateSetting;             // A number that identifies a descriptor with
                                           // alternate settings for this bInterfaceNumber.

    uint8_t bNumEndpoints;                 // Number of endpoints supported not counting
                                           // endpoint zero

    uint8_t bInterfaceClass;               // Class code

    uint8_t bInterfaceSubclass;            // Subclass code

    uint8_t bInterfaceProtocol;            // Protocol code

    uint8_t iInterface;                    // Index of string descriptor for the interface

} USB_INTERFACE_DESCRIPTOR_STRUCT;


typedef struct __attribute__((packed))
{
    uint8_t bLength;                    // Descriptor size in bytes (07h)

    uint8_t bDescriptorType;            // The constant ENDPOINT (05h)

    uint8_t bEndpointAddress;           // Endpoint number and direction

    uint8_t bmAttributes;               // Transfer type and supplementary information

    uint16_t wMaxPacketSize;            // Maximum packet size supported

    uint8_t bInterval;                  // Service interval or NAK rate

} USB_ENDPOINT_DESCRIPTOR_STRUCT;

void Usb0ISR( void );

void usb_init( void );

#ifdef __cplusplus
}
#endif  /* extern "C" */

#endif /* __USB_H__ */
